@@ -18,7 +18,6 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
-#include <assert.h>
 #include <ctype.h>
 
 #include "byte_order.h"
@@ -570,18 +569,6 @@ static void log_va(int level, const char* format, va_list args)
 	}
 }
 
-/**
- * Print a formatted message.
- * @param format the format of the message
- */
-static void log_message(char* format, ...)
-{
-	va_list vl;
-	va_start(vl, format);
-	log_va(0, format, vl);
-	va_end(vl);
-}
-
 /**
  * Print a formatted debug message.
  * The message is shown only in verbose mode (-v).
@@ -608,14 +595,54 @@ static void dbg2(const char* format, ...)
 	va_end(vl);
 }
 
-/*=========================================================================*
- *               Functions for calculating a message digest                *
- *=========================================================================*/
-
 /**
  * The total number of errors
  */
-static int g_errors = 0;
+static int g_errors_count = 0;
+
+/**
+ * Print a formatted error message.
+ * @param line the line number of the reported error
+ * @param format the format of the message
+ */
+static void log_error_impl(int line, const char* format, ...)
+{
+	va_list vl;
+	va_start(vl, format);
+	printf("%s:%d: error: ", __FILE__, line);
+	log_va(0, format, vl);
+	va_end(vl);
+	g_errors_count++;
+}
+
+#define log_error(msg) \
+	(log_error_impl(__LINE__, (msg)))
+#define log_error1(msg, a) \
+	(log_error_impl(__LINE__, (msg), (a)))
+#define log_error2(msg, a, b) \
+	(log_error_impl(__LINE__, (msg), (a), (b)))
+#define log_error3(msg, a, b, c) \
+	(log_error_impl(__LINE__, (msg), (a), (b), (c)))
+#define log_error4(msg, a, b, c, d) \
+	(log_error_impl(__LINE__, (msg), (a), (b), (c), (d)))
+
+#define CHECK_NOOP {}
+#define CHECK_IMPL(failed, cmd, msg) \
+	if (failed) { \
+		log_error(msg); \
+		cmd; \
+	}
+#define CHECK_EQ(a, b, msg) CHECK_IMPL((a) != (b), CHECK_NOOP, msg)
+#define CHECK_NE(a, b, msg) CHECK_IMPL((a) == (b), CHECK_NOOP, msg)
+#define CHECK_TRUE(condition, msg) CHECK_IMPL(!(condition), CHECK_NOOP, msg)
+#define REQUIRE_EQ(a, b, msg) CHECK_IMPL((a) != (b), return, msg)
+#define REQUIRE_NE(a, b, msg) CHECK_IMPL((a) == (b), return, msg)
+#define REQUIRE_TRUE(condition, msg) CHECK_IMPL(!(condition), return, msg)
+
+
+/*=========================================================================*
+ *               Functions for calculating a message digest                *
+ *=========================================================================*/
 
 enum ChunkedDataBitFlags {
 	CHDT_NO_FLAGS = 0,
@@ -637,12 +664,20 @@ static char* hash_data_by_chunks(unsigned hash_id, const char* data, size_t chun
 	struct rhash_context* ctx;
 	size_t left, size;
 	static char out[130];
-	assert(rhash_get_hash_length(hash_id) < 130);
+
+	if (rhash_get_hash_length(hash_id) >= (int)sizeof(out)) {
+		log_error("too big hash length\n");
+		return "";
+	}
 
 	ctx = rhash_init(hash_id);
-
-	if ((hash_id & RHASH_BTIH) && (flags & CHDT_SET_FILENAME)) {
-		rhash_torrent_add_file(ctx, "test.txt", (unsigned long long)total_size);
+	if (!ctx) {
+		log_error1("got NULL context for hash_id=0x%08x\n", hash_id);
+		return "";
+	}
+	if ((hash_id == RHASH_BTIH || hash_id == RHASH_ALL_HASHES) && (flags & CHDT_SET_FILENAME)) {
+		CHECK_NE(0, rhash_torrent_add_file(ctx, "test.txt", (unsigned long long)total_size),
+			"failed to add filename");
 	}
 	if (!!(flags & CHDT_REPEAT_SMALL_CHUNK)) {
 		/* repeat the small chunk of data until the total_size is reached */
@@ -683,10 +718,9 @@ static void assert_hash_long_msg(unsigned hash_id, const char* data, size_t chun
 	if (strcmp(result, hash) != 0) {
 		const char* hash_name = rhash_get_name(hash_id); /* the hash function name */
 		if (msg_name)
-			log_message("failed: %s(%s) = %s, expected %s\n", hash_name, msg_name, result, hash);
+			log_error4("%s(%s) = %s, expected %s\n", hash_name, msg_name, result, hash);
 		else
-			log_message("failed: %s(\"%s\") = %s, expected %s\n", hash_name, data, result, hash);
-		g_errors++;
+			log_error4("%s(\"%s\") = %s, expected %s\n", hash_name, data, result, hash);
 	}
 }
 
@@ -902,7 +936,7 @@ static void test_results_consistency(void)
 
 	for (i = 0, hash_id = 1; (hash_id & RHASH_ALL_HASHES); hash_id <<= 1, i++) {
 		digest_size = rhash_get_digest_size(hash_id);
-		assert(digest_size < 70);
+		REQUIRE_TRUE(digest_size < 70, "too big digest size\n");
 
 		ctx = rhash_init(hash_id);
 
@@ -918,8 +952,7 @@ static void test_results_consistency(void)
 		rhash_free(ctx);
 
 		if (memcmp(res1, res2, digest_size) != 0) {
-			log_message("failed: inconsistent %s(\"%s\") hash results\n", rhash_get_name(hash_id), msg);
-			g_errors++;
+			log_error2("inconsistent %s(\"%s\") hash results\n", rhash_get_name(hash_id), msg);
 		}
 	}
 }
@@ -930,13 +963,18 @@ static void test_results_consistency(void)
 static void test_unaligned_messages_consistency(void)
 {
 	int start, alignment_size;
-	unsigned hash_id;
+	unsigned all_hash_ids[RHASH_HASH_COUNT];
+	size_t count = rhash_get_all_algorithms(RHASH_HASH_COUNT, all_hash_ids);
+	size_t i;
 	dbg("test unaligned messages consistency\n");
+	REQUIRE_NE(RHASH_ERROR, count, "failed to get all algorithms\n");
 
 	/* loop by hash algorithms */
-	for (hash_id = 1; (hash_id & RHASH_ALL_HASHES); hash_id <<= 1) {
+	for (i = 0; i < count; i++) {
+		unsigned hash_id = all_hash_ids[i];
 		char expected_hash[130];
-		assert(rhash_get_digest_size(hash_id) < (int)sizeof(expected_hash));
+		REQUIRE_NE(0, hash_id, "bad hash_id == 0\n");
+		REQUIRE_TRUE(rhash_get_digest_size(hash_id) < (int)sizeof(expected_hash), "too big digest size\n");
 
 		alignment_size = (hash_id & (RHASH_TTH | RHASH_TIGER | RHASH_WHIRLPOOL | RHASH_SHA512) ? 8 : 4);
 
@@ -965,15 +1003,18 @@ static void test_unaligned_messages_consistency(void)
 static void test_chunk_size_consistency(void)
 {
 	char buffer[8192];
-	unsigned hash_id;
-	size_t i;
+	unsigned all_hash_ids[RHASH_HASH_COUNT];
+	size_t count = rhash_get_all_algorithms(RHASH_HASH_COUNT, all_hash_ids);
+	size_t i , j;
 	dbg("test chunk size consistency\n");
+	REQUIRE_NE(RHASH_ERROR, count, "failed to get all algorithms\n");
 
 	for (i = 0; i < sizeof(buffer); i++)
 		buffer[i] = (char)(unsigned char)(i % 255);
 
 	/* loop by hash algorithms */
-	for (hash_id = 1; (hash_id & RHASH_ALL_HASHES); hash_id <<= 1) {
+	for (j = 0; j < count; j++) {
+		unsigned hash_id = all_hash_ids[j];
 		char expected_hash[130];
 		strcpy(expected_hash, hash_data(hash_id, buffer, sizeof(buffer), 0)); /* save hash value */
 		for (i = 0; i < 2; i++) {
@@ -1002,12 +1043,11 @@ static void test_context_alignment(void)
 		char* context_ptr;
 		hash_ids[i] = RHASH_MD5;
 		ctx = rhash_init_multi(count, hash_ids);
-		assert(!!ctx);
-		context_ptr = (char*)rhash_get_context_ptr(ctx, RHASH_MD5);
+		REQUIRE_TRUE(ctx, "NULL rhash context\n");
+		rhash_get_context(ctx, RHASH_MD5, &context_ptr);
 		if (((context_ptr - (char*)0) & aligner) != 0) {
-			log_message("error: wrong aligment %d of pointer %p for the %2d-th context (rhash = %p)\n",
+			log_error4("wrong aligment %d of pointer %p for the %2d-th context (rhash = %p)\n",
 				(int)((context_ptr - (char*)0) & aligner), context_ptr, i, ctx);
-			g_errors++;
 		}
 		rhash_free(ctx);
 		hash_ids[i] = RHASH_CRC32;
@@ -1015,25 +1055,20 @@ static void test_context_alignment(void)
 }
 
 /**
- * Verify processor endianness detected at compile-time against
- * with the actual CPU endianness in runtime.
+ * Verify processor endianness detected at compile time
+ * against the actual CPU endianness.
  */
 static void test_endianness(void)
 {
 	unsigned tmp = 1;
-	if (*(char*)&tmp == IS_BIG_ENDIAN) {
-		log_message("error: wrong endianness detected at compile time\n");
-		g_errors++;
-	}
+	CHECK_EQ(*(char*)&tmp, IS_LITTLE_ENDIAN, "wrong endianness detected at compile time\n");
 }
 
 static void test_version_sanity(void)
 {
 	unsigned version = rhash_get_version();
-	if (!(version & 0xff000000) || (version & 0xf0e0e0e0)) {
-		log_message("error: wrong librhash version: %x\n", version);
-		g_errors++;
-	}
+	if (!(version & 0xff000000) || (version & 0xf0e0e0e0))
+		log_error1("wrong librhash version: %x\n", version);
 }
 
 /**
@@ -1041,21 +1076,126 @@ static void test_version_sanity(void)
  */
 static void test_generic_assumptions(void)
 {
-	unsigned mask = (1u << RHASH_HASH_COUNT) - 1u;
 	dbg("test generic assumptions\n");
-	if (mask != RHASH_ALL_HASHES) {
-		log_message("error: wrong algorithms count %d for the mask 0x%x\n", RHASH_HASH_COUNT, RHASH_ALL_HASHES);
-		g_errors++;
+	if (RHASH_HASH_COUNT < rhash_popcount(RHASH_ALL_HASHES)) {
+		log_error2("wrong algorithms count %d for low mask 0x%x\n", RHASH_HASH_COUNT, RHASH_ALL_HASHES);
 	}
-
+	CHECK_TRUE(!(RHASH_EXTENDED_BIT & RHASH_ALL_HASHES), "bad RHASH_ALL_HASHES value");
 	test_endianness();
 	test_version_sanity();
 }
 
+static void check_openssl_ids(size_t count, unsigned* hash_ids)
+{
+	size_t i;
+	for (i = 0; i < count; i++) {
+		unsigned id = hash_ids[i];
+		if ((id & (id - 1)) != 0 || (id & 0x000f060e) == 0)
+			log_error2("got bad openssl algorithm id = %08x (%s)\n", id, rhash_get_name(id));
+	}
+}
+
+/**
+ * Test getting of ids.
+ */
+static void test_openssl_getters(void)
+{
+	const size_t total = (size_t)RHASH_HASH_COUNT;
+	unsigned hash_ids[RHASH_HASH_COUNT];
+	size_t scount, acount, ecount;
+
+	dbg2("- test openssl calls\n");
+	scount = rhash_get_openssl_supported(0, NULL);
+	REQUIRE_TRUE(scount <= total, "openssl: supported count is greater than the total slgorithm count\n");
+	CHECK_EQ(scount, rhash_get_openssl_supported(0, hash_ids), "wrong openssl algorithms count\n");
+	CHECK_EQ(scount, rhash_get_openssl_supported(total, NULL), "wrong openssl algorithms count\n");
+	if (scount > 1)
+		CHECK_EQ(RHASH_ERROR, rhash_get_openssl_supported(1, hash_ids), "RHASH_ERROR expected\n");
+	CHECK_EQ(scount, rhash_get_openssl_supported(scount, hash_ids), "wrong openssl algorithms count\n");
+	check_openssl_ids(scount, hash_ids);
+	acount = rhash_get_openssl_available(0, NULL);
+	CHECK_TRUE(acount <= scount, "openssl: available count is greater than supported count\n");
+	CHECK_EQ(acount, rhash_get_openssl_available(0, hash_ids), "wrong openssl algorithms count\n");
+	CHECK_EQ(acount, rhash_get_openssl_available(total, NULL), "wrong openssl algorithms count\n");
+	CHECK_EQ(acount, rhash_get_openssl_available(acount, hash_ids), "wrong openssl algorithms count\n");
+	check_openssl_ids(acount, hash_ids);
+	ecount = rhash_get_openssl_enabled(0, NULL);
+	CHECK_TRUE(ecount <= acount, "openssl: enabled count is greater than available count\n");
+	CHECK_EQ(ecount, rhash_get_openssl_enabled(0, hash_ids), "wrong openssl algorithms count\n");
+	CHECK_EQ(ecount, rhash_get_openssl_enabled(total, NULL), "wrong openssl algorithms count\n");
+	CHECK_EQ(ecount, rhash_get_openssl_enabled(ecount, hash_ids), "wrong openssl algorithms count\n");
+	check_openssl_ids(ecount, hash_ids);
+	/* test changing the list of enabled openssl algorithms */
+	CHECK_EQ(RHASH_ERROR, rhash_set_openssl_enabled(total, NULL), "RHASH_ERROR expected\n");
+	CHECK_EQ(ecount, rhash_get_openssl_enabled(0, NULL), "enabled algorithms changed\n");
+	REQUIRE_EQ(total, rhash_get_all_algorithms(total, hash_ids), "failed to get all algorithms\n");
+	CHECK_EQ(0, rhash_set_openssl_enabled(total, hash_ids), "failed to enable all openssl algorithms\n");
+	CHECK_EQ(acount, rhash_get_openssl_enabled(0, NULL), "all available algorithms must be enabled\n");
+	CHECK_EQ(0, rhash_set_openssl_enabled(0, NULL), "disabling algorithms must work without ids array\n");
+	CHECK_EQ(0, rhash_get_openssl_enabled(0, NULL), "openssl algorithms were not disabled\n");
+	REQUIRE_EQ(acount, rhash_get_openssl_available(acount, hash_ids), "failed to get available algorithms\n");
+	CHECK_EQ(0, rhash_set_openssl_enabled(acount, hash_ids), "failed to enable all openssl algorithms\n");
+	CHECK_EQ(acount, rhash_get_openssl_enabled(0, NULL), "not all available algorithms were anbled\n");
+	CHECK_EQ(0, rhash_set_openssl_enabled(0, hash_ids), "failed to disable openssl algorithms\n");
+	CHECK_EQ(0, rhash_get_openssl_enabled(0, NULL), "openssl algorithms were not disabled\n");
+}
+
+/**
+ * Test getting of ids.
+ */
+static void test_id_getters(void)
+{
+	const size_t count = (size_t)RHASH_HASH_COUNT;
+	unsigned hash_ids[RHASH_HASH_COUNT];
+	unsigned hash_ids2[RHASH_HASH_COUNT];
+	rhash ctx;
+	dbg("test algorithms getters\n");
+	CHECK_EQ(count, rhash_get_all_algorithms(0, hash_ids), "incorrect number of supported algorithms\n");
+	CHECK_EQ(count, rhash_get_all_algorithms(1, 0), "incorrect number of supported algorithms\n");
+	CHECK_EQ(RHASH_ERROR, rhash_get_all_algorithms(1, hash_ids), "RHASH_ERROR expected for small buffer\n");
+	CHECK_EQ(RHASH_ERROR, rhash_get_all_algorithms(count - 1, hash_ids), "RHASH_ERROR expected for small buffer\n");
+	REQUIRE_EQ(count, rhash_get_all_algorithms(count, hash_ids), "failed to get ids of supported algorithms\n");
+	ctx = rhash_init_multi(count, hash_ids);
+	REQUIRE_NE(0, ctx, "got invalid context\n");
+	CHECK_EQ(count, rhash_get_ctx_algorithms(ctx, 0, hash_ids2), "incorrect number of context algorithms\n");
+	CHECK_EQ(count, rhash_get_ctx_algorithms(ctx, 1, 0), "incorrect number of context algorithms\n");
+	CHECK_EQ(RHASH_ERROR, rhash_get_ctx_algorithms(ctx, 1, hash_ids2), "RHASH_ERROR expected for small buffer\n");
+	CHECK_EQ(RHASH_ERROR, rhash_get_ctx_algorithms(ctx, count - 1, hash_ids2), "RHASH_ERROR expected for small buffer\n");
+	REQUIRE_EQ(count, rhash_get_ctx_algorithms(ctx, count, hash_ids2), "failed to get ids of context algorithms\n");
+	CHECK_EQ(0, memcmp(hash_ids, hash_ids2, sizeof(hash_ids)), "unexpected algorithms ids\n");
+	rhash_free(ctx);
+
+#if defined(USE_OPENSSL) || defined(OPENSSL_RUNTIME)
+	REQUIRE_NE(0, rhash_is_openssl_supported(), "openssl must be on\n");
+#else
+	REQUIRE_EQ(0, rhash_is_openssl_supported(), "openssl must be off\n");
+#endif
+	test_openssl_getters();
+}
+
+static void test_get_context(void)
+{
+	unsigned hash_ids[2] = { RHASH_CRC32, RHASH_HAS160 };
+	void* context_ptr = NULL;
+	rhash ctx = rhash_init_multi(2, hash_ids);
+	REQUIRE_TRUE(ctx, "NULL rhash context\n");
+	CHECK_EQ(RHASH_ERROR, rhash_get_context(ctx, RHASH_MD5, &context_ptr), "RHASH_ERROR expected\n");
+	CHECK_EQ(RHASH_ERROR, rhash_get_context(ctx, RHASH_SHA1, &context_ptr), "RHASH_ERROR expected\n");
+	CHECK_EQ(0, rhash_get_context(ctx, RHASH_CRC32, &context_ptr), "failed to get context pointer\n");
+	REQUIRE_NE(NULL, context_ptr, "NULL context pointer\n");
+	CHECK_EQ(0, *(uint32_t*)context_ptr, "wrong crc32 value\n");
+	rhash_update(ctx, "abcd", 4);
+	CHECK_EQ(0, rhash_get_context(ctx, RHASH_HAS160, &context_ptr), "failed to get context pointer\n");
+	REQUIRE_NE(NULL, context_ptr, "NULL context pointer\n");
+	CHECK_EQ(0, memcmp("abcd", context_ptr, 4), "wrong message buffer content in hash context\n");
+	CHECK_EQ(4, *((uint64_t*)context_ptr + 8), "wrong message length in hash context\n");
+	rhash_free(ctx);
+}
+
 static void test_import_export(void)
 {
 #if !defined(NO_IMPORT_EXPORT)
-	unsigned hash_mask = RHASH_ALL_HASHES;
+	unsigned export_id = RHASH_ALL_HASHES;
 	uint8_t data[241];
 	size_t i;
 	size_t min_sizes[3] = { 0, 1024, 8192 };
@@ -1068,12 +1208,17 @@ static void test_import_export(void)
 		size_t required_size;
 		size_t exported_size;
 		void* exported_data;
-		rhash ctx = rhash_init(hash_mask);
+		unsigned imported_ids[RHASH_HASH_COUNT];
+		size_t imported_ids_count;
+		size_t j;
+		rhash ctx = rhash_init(export_id);
 		rhash imported_ctx;
-		unsigned hash_id;
+		size_t exported_ids_count = rhash_get_ctx_algorithms(ctx, 0, 0);
+		CHECK_TRUE(exported_ids_count > 0, "wrong number of exported algorithms");
+
 		for (; size < min_size; size += sizeof(data))
 			rhash_update(ctx, data, sizeof(data));
-		if ((hash_mask & RHASH_BTIH) != 0) {
+		if (export_id == RHASH_BTIH || export_id == RHASH_ALL_HASHES) {
 			rhash_torrent_set_program_name(ctx, "test");
 			rhash_torrent_add_announce(ctx, "url1");
 			rhash_torrent_add_announce(ctx, "url2");
@@ -1086,23 +1231,20 @@ static void test_import_export(void)
 		dbg2("- call rhash_export NULL\n");
 		required_size = rhash_export(ctx, NULL, 0);
 		if (!required_size) {
-			log_message("error: rhash_export failed for block size=%u\n", (unsigned)size);
-			g_errors++;
+			log_error1("rhash_export failed for block size=%u\n", (unsigned)size);
 			return;
 		}
 		dbg2("- call rhash_export ctx\n");
 		exported_data = malloc(required_size);
 		exported_size = rhash_export(ctx, exported_data, required_size);
 		if (exported_size != required_size) {
-			log_message("error: rhash_export failed: %u != %u\n", (unsigned)exported_size, (unsigned)required_size);
-			g_errors++;
+			log_error2("rhash_export failed: %u != %u\n", (unsigned)exported_size, (unsigned)required_size);
 			return;
 		}
 		dbg2("- call rhash_import\n");
 		imported_ctx = rhash_import(exported_data, required_size);
 		if (!imported_ctx) {
-			log_message("error: rhash_import failed for block size=%u\n", (unsigned)size);
-			g_errors++;
+			log_error1("rhash_import failed for block size=%u\n", (unsigned)size);
 			return;
 		}
 		free(exported_data);
@@ -1110,20 +1252,20 @@ static void test_import_export(void)
 		rhash_final(ctx, 0);
 		dbg2("- call rhash_final imported_ctx\n");
 		rhash_final(imported_ctx, 0);
-		dbg2("- verify results\n");
 		exported_data = NULL;
-		for (hash_id = 1; hash_id < hash_mask; hash_id <<= 1) {
-			if ((hash_id & hash_mask) != 0) {
-				static char out1[240], out2[240];
-				rhash_print(out1, ctx, hash_id, RHPR_UPPERCASE);
-				rhash_print(out2, imported_ctx, hash_id, RHPR_UPPERCASE);
-				if (strcmp(out1, out2) != 0) {
-					const char* hash_name = rhash_get_name(hash_id);
-					log_message("error: import failed, wrong hash %s != %s for %s,  block size=%u\n",
-						out1, out2, hash_name, (unsigned)size);
-					g_errors++;
-					return;
-				}
+		imported_ids_count = rhash_get_ctx_algorithms(ctx, RHASH_HASH_COUNT, imported_ids);
+		CHECK_EQ(exported_ids_count, imported_ids_count, "wrong number of imported algorithms");
+		dbg2("- verify results\n");
+		for (j = 0; j < imported_ids_count; j++) {
+			unsigned hash_id = imported_ids[j];
+			static char out1[240], out2[240];
+			REQUIRE_NE(0, rhash_print(out1, ctx, hash_id, RHPR_UPPERCASE), "rhash_print failed");
+			REQUIRE_NE(0, rhash_print(out2, imported_ctx, hash_id, RHPR_UPPERCASE), "rhash_print failed");
+			if (strcmp(out1, out2) != 0) {
+				const char* hash_name = rhash_get_name(hash_id);
+				log_error4("import failed, wrong hash %s != %s for %s,  block size=%u\n",
+					out1, out2, hash_name, (unsigned)size);
+				return;
 			}
 		}
 		rhash_free(ctx);
@@ -1140,23 +1282,25 @@ static void test_import_export(void)
 static void assert_magnet(const char* expected,
 	rhash ctx, unsigned mask, int flags)
 {
-	static char out[240];
+	static char out[2800];
 	const char* path = (flags & TEST_PATH ? "test.txt" : NULL);
-	size_t size;
+	size_t size_calculated = rhash_print_magnet(NULL, path, ctx, mask, flags);
+	size_t size_printed;
+	REQUIRE_TRUE(size_calculated < sizeof(out), "too small buffer for magnet link\n");
+
 	flags &= ~TEST_PATH;
-	size = rhash_print_magnet(out, path, ctx, mask, flags);
+	size_printed = rhash_print_magnet(out, path, ctx, mask, flags);
 
 	if (expected && strcmp(expected, out) != 0) {
-		log_message("error: \"%s\" != \"%s\"\n", expected, out);
-		g_errors++;
+		log_error2("\"%s\" != \"%s\"\n", expected, out);
 	} else {
-		size_t size2 = strlen(out) + 1;
-		if (size != size2) {
-			log_message("error: rhash_print_magnet returns wrong length %d != %d for \"%s\"\n", (int)size, (int)size2, out);
-			g_errors++;
-		} else if (size != (size2 = rhash_print_magnet(NULL, path, ctx, mask, flags))) {
-			log_message("error: rhash_print_magnet(NULL, ...) returns wrong length %d != %d for \"%s\"\n", (int)size2, (int)size, out);
-			g_errors++;
+		size_t length = strlen(out) + 1;
+		if (size_printed != length) {
+			log_error3("rhash_print_magnet returns wrong length %u != %u for \"%s\"\n",
+				(unsigned)size_printed, (unsigned)length, out);
+		} else if (size_printed != size_calculated) {
+			log_error3("rhash_print_magnet(NULL, ...) returns wrong length %u != %u for \"%s\"\n",
+				(unsigned)size_calculated, (unsigned)size_printed, out);
 		}
 	}
 }
@@ -1166,7 +1310,9 @@ static void assert_magnet(const char* expected,
  */
 static void test_magnet(void)
 {
-	unsigned bit;
+	unsigned all_hash_ids[RHASH_HASH_COUNT];
+	size_t count = rhash_get_all_algorithms(RHASH_HASH_COUNT, all_hash_ids);
+	size_t i;
 	rhash ctx;
 	dbg("test magnet link\n");
 	ctx	= rhash_init(RHASH_ALL_HASHES);
@@ -1179,10 +1325,11 @@ static void test_magnet(void)
 		ctx, RHASH_ED2K | RHASH_AICH | RHASH_SHA1 | RHASH_BTIH, RHPR_NO_MAGNET);
 
 	/* verify length calculation for all hashes */
-	for (bit = 1; bit < RHASH_ALL_HASHES; bit <<= 1) {
-		assert_magnet(NULL, ctx, bit, RHPR_FILESIZE | RHPR_NO_MAGNET);
+	for (i = 0; i < count; i++) {
+		unsigned hash_id = all_hash_ids[i];
+		assert_magnet(NULL, ctx, hash_id, RHPR_FILESIZE | RHPR_NO_MAGNET);
 	}
-
+	assert_magnet(NULL, ctx, RHASH_ALL_HASHES, RHPR_FILESIZE | RHPR_NO_MAGNET);
 	rhash_free(ctx);
 }
 
@@ -1213,31 +1360,33 @@ static unsigned find_hash(const char* name)
  */
 static void print_openssl_status(void)
 {
-	rhash_uptr_t available = rhash_get_openssl_available_mask();
-	rhash_uptr_t supported = rhash_get_openssl_supported_mask();
-	int has_openssl = rhash_is_openssl_supported();
+	unsigned available[RHASH_HASH_COUNT];
+	unsigned supported[RHASH_HASH_COUNT];
+	size_t available_count, supported_count;
 
-	printf("OpenSSL %s", (has_openssl ? "supported" : "not supported"));
-	if (has_openssl && available != RHASH_ERROR)
-	{
-		printf(", %s", (available ? "loaded" : "not loaded"));
-		if (available)
-		{
-			unsigned hash_id;
-			printf(":");
-			available &= RHASH_ALL_HASHES;
-			for (hash_id = 1; hash_id <= available; hash_id <<= 1) {
-				if (!!(hash_id & available))
-					printf(" %s", rhash_get_name(hash_id));
-			}
-			supported &= (~available & RHASH_ALL_HASHES);
-			for (hash_id = 1; hash_id <= supported; hash_id <<= 1) {
-				if (!!(hash_id & supported))
-					printf(" -%s", rhash_get_name(hash_id));
-			}
+	if (!rhash_is_openssl_supported()) {
+		printf("OpenSSL is not supported\n");
+		return;
+	}
+	supported_count = rhash_get_openssl_supported(RHASH_HASH_COUNT, supported);
+	available_count = rhash_get_openssl_available(RHASH_HASH_COUNT, available);
+	REQUIRE_NE(RHASH_ERROR, supported_count, "failed to load supported openssl algorithms\n");
+	REQUIRE_NE(RHASH_ERROR, available_count, "failed to load available openssl algorithms\n");
+
+	printf("OpenSSL is supported, %s",
+		(available_count ? "loaded:" : "not loaded\n"));
+	if (available_count > 0) {
+		size_t i, j;
+		for (i = 0; i < available_count; i++)
+			printf(" %s", rhash_get_name(available[i]));
+		for (i = 0; i < supported_count; i++) {
+			unsigned id = supported[i];
+			for (j = 0; j < available_count && available[j] != id; j++);
+			if (j == available_count)
+				printf(" -%s", rhash_get_name(id));
 		}
+		printf("\n");
 	}
-	printf("\n");
 }
 
 /**
@@ -1254,13 +1403,13 @@ int main(int argc, char* argv[])
 	int i;
 
 	for (i = 1; i < argc; i++) {
-		if (strcmp(argv[i], "--info") == 0) {
+		if (!strcmp(argv[i], "--info") || !strcmp(argv[i], "-i")) {
 			print_info = 1;
 		}
-		else if (strcmp(argv[i], "--verbose") == 0 || strcmp(argv[i], "-v") == 0) {
+		else if (!strcmp(argv[i], "--verbose") || !strcmp(argv[i], "-v")) {
 			g_verbose++;
 		}
-		else if (strcmp(argv[i], "--speed") == 0) {
+		else if (!strcmp(argv[i], "--speed") || !strcmp(argv[i], "-s")) {
 			test_speed = 1;
 			if ((i + 1) < argc && argv[i + 1][0] != '-') {
 				hash_id = find_hash(argv[i + 1]);
@@ -1273,9 +1422,10 @@ int main(int argc, char* argv[])
 		}
 		else {
 			printf("Options:\n"
-				"--verbose Be verbose.\n"
-				"--info Print library info\n"
-				"--speed [HASH_NAME] Benchmark given hash algorithm\n");
+				"-h, --help Print help.\n"
+				"-v, --verbose Be verbose.\n"
+				"-i, --info Print library info\n"
+				"-s, --speed [HASH_NAME] Benchmark given hash algorithm\n");
 			return 1;
 		}
 	}
@@ -1297,13 +1447,15 @@ int main(int argc, char* argv[])
 		test_unaligned_messages_consistency();
 		test_chunk_size_consistency();
 		test_context_alignment();
+		test_id_getters();
+		test_get_context();
 		test_import_export();
 		test_magnet();
-		if (g_errors == 0)
+		if (g_errors_count == 0)
 			printf("All sums are working properly!\n");
 		fflush(stdout);
 	}
-	if (g_errors > 0)
+	if (g_errors_count > 0)
 		printf("%s", compiler_flags);
-	return (g_errors == 0 ? 0 : 1);
+	return (g_errors_count == 0 ? 0 : 1);
 }
